@@ -22,9 +22,10 @@ type ArtifactsDownloaderCommand struct {
 	DirectDownload bool `long:"direct-download" env:"FF_USE_DIRECT_DOWNLOAD" description:"Support direct download for data stored externally to GitLab"`
 }
 
-func (c *ArtifactsDownloaderCommand) directDownloadFlag() *bool {
+func (c *ArtifactsDownloaderCommand) directDownloadFlag(retry int) *bool {
 	// We want to send `?direct_download=true`
-	if c.DirectDownload {
+	// Use direct download only on a first attempt
+	if c.DirectDownload && retry == 0 {
 		return &c.DirectDownload
 	}
 
@@ -32,8 +33,8 @@ func (c *ArtifactsDownloaderCommand) directDownloadFlag() *bool {
 	return nil
 }
 
-func (c *ArtifactsDownloaderCommand) download(file string) error {
-	switch c.network.DownloadArtifacts(c.JobCredentials, file, c.directDownloadFlag()) {
+func (c *ArtifactsDownloaderCommand) download(file string, retry int) error {
+	switch c.network.DownloadArtifacts(c.JobCredentials, file, c.directDownloadFlag(retry)) {
 	case common.DownloadSucceeded:
 		return nil
 	case common.DownloadNotFound:
@@ -66,8 +67,8 @@ func (c *ArtifactsDownloaderCommand) Execute(context *cli.Context) {
 	defer os.Remove(file.Name())
 
 	// Download artifacts file
-	err = c.doRetry(func() error {
-		return c.download(file.Name())
+	err = c.doRetry(func(retry int) error {
+		return c.download(file.Name(), retry)
 	})
 	if err != nil {
 		logrus.Fatalln(err)
