@@ -20,6 +20,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 	"gitlab.com/gitlab-org/gitlab-runner/session/terminal"
 )
@@ -31,52 +32,6 @@ func init() {
 	RegisterShell(&s)
 }
 
-func TestBuildRun(t *testing.T) {
-	e := MockExecutor{}
-	defer e.AssertExpectations(t)
-
-	p := MockExecutorProvider{}
-	defer p.AssertExpectations(t)
-
-	// Create executor only once
-	p.On("CanCreate").Return(true).Once()
-	p.On("GetDefaultShell").Return("bash").Once()
-	p.On("GetFeatures", mock.Anything).Return(nil).Twice()
-
-	p.On("Create").Return(&e).Once()
-
-	// We run everything once
-	e.On("Prepare", mock.Anything, mock.Anything, mock.Anything).Return(nil).Once()
-	e.On("Finish", nil).Once()
-	e.On("Cleanup").Once()
-
-	// Run script successfully
-	e.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
-	e.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(nil).Once()
-
-	RegisterExecutorProvider("build-run-test", &p)
-
-	successfulBuild, err := GetSuccessfulBuild()
-	assert.NoError(t, err)
-	build := &Build{
-		JobResponse: successfulBuild,
-		Runner: &RunnerConfig{
-			RunnerSettings: RunnerSettings{
-				Executor: "build-run-test",
-			},
-		},
-	}
-	err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
-	assert.NoError(t, err)
-}
-
 func TestBuildPredefinedVariables(t *testing.T) {
 	for _, rootDir := range []string{"/root/dir1", "/root/dir2"} {
 		t.Run(rootDir, func(t *testing.T) {
@@ -115,7 +70,7 @@ func runSuccessfulMockBuild(t *testing.T, prepareFn func(options ExecutorPrepare
 	e.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Return(nil).Once()
+	e.On("Run", matchBuildStage("step_script")).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(nil).Once()
@@ -138,6 +93,16 @@ func runSuccessfulMockBuild(t *testing.T, prepareFn func(options ExecutorPrepare
 	return build
 }
 
+func matchBuildStage(buildStage BuildStage) interface{} {
+	return mock.MatchedBy(func(cmd ExecutorCommand) bool {
+		return cmd.Stage == buildStage
+	})
+}
+
+func TestBuildRun(t *testing.T) {
+	runSuccessfulMockBuild(t, func(options ExecutorPrepareOptions) error { return nil })
+}
+
 func TestJobImageExposed(t *testing.T) {
 	tests := map[string]struct {
 		image           string
@@ -217,7 +182,7 @@ func TestBuildRunNoModifyConfig(t *testing.T) {
 	e.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Return(nil).Once()
+	e.On("Run", matchBuildStage("step_script")).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(nil).Once()
@@ -357,7 +322,46 @@ func TestPrepareFailureOnBuildError(t *testing.T) {
 		},
 	}
 	err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
-	assert.IsType(t, err, &BuildError{})
+
+	expectedErr := new(BuildError)
+	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+}
+
+func TestPrepareEnvironmentFailure(t *testing.T) {
+	testErr := errors.New("test-err")
+
+	e := new(MockExecutor)
+	defer e.AssertExpectations(t)
+
+	p := new(MockExecutorProvider)
+	defer p.AssertExpectations(t)
+
+	p.On("CanCreate").Return(true).Once()
+	p.On("GetDefaultShell").Return("bash").Once()
+	p.On("GetFeatures", mock.Anything).Return(nil).Twice()
+	p.On("Create").Return(e).Once()
+
+	e.On("Prepare", mock.Anything, mock.Anything, mock.Anything).Return(nil).Once()
+	e.On("Cleanup").Once()
+	e.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+	e.On("Run", matchBuildStage(BuildStagePrepare)).Return(testErr).Once()
+	e.On("Finish", mock.Anything).Once()
+
+	RegisterExecutorProvider("build-run-prepare-environment-failure-on-build-error", p)
+
+	successfulBuild, err := GetSuccessfulBuild()
+	assert.NoError(t, err)
+	build := &Build{
+		JobResponse: successfulBuild,
+		Runner: &RunnerConfig{
+			RunnerSettings: RunnerSettings{
+				Executor: "build-run-prepare-environment-failure-on-build-error",
+			},
+		},
+	}
+
+	err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
+	assert.True(t, errors.Is(err, testErr))
 }
 
 func TestJobFailure(t *testing.T) {
@@ -382,7 +386,8 @@ func TestJobFailure(t *testing.T) {
 	// Succeed a build script
 	thrownErr := &BuildError{Inner: errors.New("test error")}
 	e.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
-	e.On("Run", mock.Anything).Return(thrownErr)
+	e.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
+	e.On("Run", mock.Anything).Return(thrownErr).Times(2)
 	e.On("Finish", thrownErr).Once()
 
 	RegisterExecutorProvider("build-run-job-failure", p)
@@ -407,7 +412,9 @@ func TestJobFailure(t *testing.T) {
 	trace.On("Fail", thrownErr, ScriptFailure).Once()
 
 	err = build.Run(&Config{}, trace)
-	require.IsType(t, &BuildError{}, err)
+
+	expectedErr := new(BuildError)
+	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
 }
 
 func TestJobFailureOnExecutionTimeout(t *testing.T) {
@@ -431,7 +438,7 @@ func TestJobFailureOnExecutionTimeout(t *testing.T) {
 
 	// Succeed a build script
 	e.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Run(func(arguments mock.Arguments) {
+	e.On("Run", matchBuildStage("step_script")).Run(func(arguments mock.Arguments) {
 		time.Sleep(2 * time.Second)
 	}).Return(nil)
 	e.On("Run", mock.Anything).Return(nil)
@@ -464,13 +471,9 @@ func TestJobFailureOnExecutionTimeout(t *testing.T) {
 	}).Once()
 
 	err = build.Run(&Config{}, trace)
-	require.IsType(t, &BuildError{}, err)
-}
 
-func matchBuildStage(buildStage BuildStage) interface{} {
-	return mock.MatchedBy(func(cmd ExecutorCommand) bool {
-		return cmd.Stage == buildStage
-	})
+	expectedErr := &BuildError{FailureReason: JobExecutionTimeout}
+	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
 }
 
 func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
@@ -497,7 +500,7 @@ func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
 	e.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Return(errors.New("build fail")).Once()
+	e.On("Run", matchBuildStage("step_script")).Return(errors.New("build fail")).Once()
 	e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
 	e.On("Finish", errors.New("build fail")).Once()
@@ -630,7 +633,7 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	e.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
-	e.On("Run", matchBuildStage(BuildStageUserScript)).Return(nil).Once()
+	e.On("Run", matchBuildStage("step_script")).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
 	e.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(errors.New("upload fail")).Once()
@@ -1157,6 +1160,46 @@ func TestStartBuild(t *testing.T) {
 	}
 }
 
+func TestSkipBuildStageFeatureFlag(t *testing.T) {
+	featureFlagValues := []string{
+		"true",
+		"false",
+	}
+
+	s := MockShell{}
+	s.On("GetName").Return("skip-build-stage-shell")
+	RegisterShell(&s)
+
+	for _, value := range featureFlagValues {
+		t.Run(value, func(t *testing.T) {
+			build := &Build{
+				Runner: &RunnerConfig{},
+				JobResponse: JobResponse{
+					Variables: JobVariables{
+						{
+							Key:   featureflags.SkipNoOpBuildStages,
+							Value: "false",
+						},
+					},
+				},
+			}
+
+			e := &MockExecutor{}
+			defer e.AssertExpectations(t)
+
+			s.On("GenerateScript", mock.Anything, mock.Anything).Return("script", ErrSkipBuildStage)
+			e.On("Shell").Return(&ShellScriptInfo{Shell: "skip-build-stage-shell"})
+
+			if !build.IsFeatureFlagOn(featureflags.SkipNoOpBuildStages) {
+				e.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
+			}
+
+			err := build.executeStage(context.Background(), BuildStageAfterScript, e)
+			assert.NoError(t, err)
+		})
+	}
+}
+
 func TestWaitForTerminal(t *testing.T) {
 	cases := []struct {
 		name                   string
@@ -1369,6 +1412,46 @@ func TestGitCleanFlags(t *testing.T) {
 	}
 }
 
+func TestGitFetchFlags(t *testing.T) {
+	tests := map[string]struct {
+		value          string
+		expectedResult []string
+	}{
+		"empty fetch flags": {
+			value:          "",
+			expectedResult: []string{"--prune", "--quiet"},
+		},
+		"use custom flags": {
+			value:          "custom-flags",
+			expectedResult: []string{"custom-flags"},
+		},
+		"use custom flags with multiple arguments": {
+			value:          "--prune --tags --quiet",
+			expectedResult: []string{"--prune", "--tags", "--quiet"},
+		},
+		"disabled": {
+			value:          "none",
+			expectedResult: []string{},
+		},
+	}
+
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			build := &Build{
+				Runner: &RunnerConfig{},
+				JobResponse: JobResponse{
+					Variables: JobVariables{
+						{Key: "GIT_FETCH_EXTRA_FLAGS", Value: test.value},
+					},
+				},
+			}
+
+			result := build.GetGitFetchFlags()
+			assert.Equal(t, test.expectedResult, result)
+		})
+	}
+}
+
 func TestDefaultVariables(t *testing.T) {
 	tests := map[string]struct {
 		jobVariables  JobVariables
@@ -1518,6 +1601,37 @@ func TestProjectUniqueName(t *testing.T) {
 	}
 }
 
+func TestBuildStages(t *testing.T) {
+	scriptOnlyBuild, err := GetRemoteSuccessfulBuild()
+	require.NoError(t, err)
+
+	multistepBuild, err := GetRemoteSuccessfulMultistepBuild()
+	require.NoError(t, err)
+
+	tests := map[string]struct {
+		jobResponse    JobResponse
+		expectedStages []BuildStage
+	}{
+		"script only build": {
+			jobResponse:    scriptOnlyBuild,
+			expectedStages: append(staticBuildStages, "step_script"),
+		},
+		"multistep build": {
+			jobResponse:    multistepBuild,
+			expectedStages: append(staticBuildStages, "step_script", "step_release"),
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			build := &Build{
+				JobResponse: tt.jobResponse,
+			}
+			assert.ElementsMatch(t, tt.expectedStages, build.BuildStages())
+		})
+	}
+}
+
 func TestBuild_GetExecutorJobSectionAttempts(t *testing.T) {
 	tests := []struct {
 		attempts         string
