@@ -103,7 +103,7 @@ type DockerConfig struct {
 	VolumesFrom                []string          `toml:"volumes_from,omitempty" json:"volumes_from" long:"volumes-from" env:"DOCKER_VOLUMES_FROM" description:"A list of volumes to inherit from another container"`
 	NetworkMode                string            `toml:"network_mode,omitempty" json:"network_mode" long:"network-mode" env:"DOCKER_NETWORK_MODE" description:"Add container to a custom network"`
 	Links                      []string          `toml:"links,omitempty" json:"links" long:"links" env:"DOCKER_LINKS" description:"Add link to another container"`
-	Services                   []Service         `toml:"services,omitempty"`
+	Services                   []Service         `toml:"services,omitempty" json:"services" description:"Add service that is started with container"`
 	WaitForServicesTimeout     int               `toml:"wait_for_services_timeout,omitzero" json:"wait_for_services_timeout" long:"wait-for-services-timeout" env:"DOCKER_WAIT_FOR_SERVICES_TIMEOUT" description:"How long to wait for service startup"`
 	AllowedImages              []string          `toml:"allowed_images,omitempty" json:"allowed_images" long:"allowed-images" env:"DOCKER_ALLOWED_IMAGES" description:"Whitelist allowed images"`
 	AllowedServices            []string          `toml:"allowed_services,omitempty" json:"allowed_services" long:"allowed-services" env:"DOCKER_ALLOWED_SERVICES" description:"Whitelist allowed services"`
@@ -233,7 +233,7 @@ type KubernetesConfig struct {
 	PodAnnotationsOverwriteAllowed   string                       `toml:"pod_annotations_overwrite_allowed" json:"pod_annotations_overwrite_allowed" long:"pod_annotations_overwrite_allowed" env:"KUBERNETES_POD_ANNOTATIONS_OVERWRITE_ALLOWED" description:"Regex to validate 'KUBERNETES_POD_ANNOTATIONS_*' values"`
 	PodSecurityContext               KubernetesPodSecurityContext `toml:"pod_security_context,omitempty" namespace:"pod-security-context" description:"A security context attached to each build pod"`
 	Volumes                          KubernetesVolumes            `toml:"volumes"`
-	Services                         []Service                    `toml:"services,omitempty" json:"services" long:"services" description:"Add service that is started with container"`
+	Services                         []Service                    `toml:"services,omitempty" json:"services" description:"Add service that is started with container"`
 }
 
 type KubernetesVolumes struct {
@@ -586,9 +586,7 @@ func (c *DockerMachine) CompilePeriods() error {
 	return nil
 }
 
-var periodTimer = func() time.Time {
-	return time.Now()
-}
+var periodTimer = time.Now
 
 // TODO: remove in 14.0: https://gitlab.com/gitlab-org/gitlab-runner/-/issues/25555
 func (c *DockerMachine) legacyCompilePeriods() error {
