@@ -67,6 +67,7 @@ type FeaturesInfo struct {
 	Masking                 bool `json:"masking"`
 	Proxy                   bool `json:"proxy"`
 	RawVariables            bool `json:"raw_variables"`
+	ArtifactsExclude        bool `json:"artifacts_exclude"`
 }
 
 type RegisterRunnerParameters struct {
@@ -203,6 +204,8 @@ type Services []Image
 
 type ArtifactPaths []string
 
+type ArtifactExclude []string
+
 type ArtifactWhen string
 
 const (
@@ -229,13 +232,14 @@ const (
 )
 
 type Artifact struct {
-	Name      string         `json:"name"`
-	Untracked bool           `json:"untracked"`
-	Paths     ArtifactPaths  `json:"paths"`
-	When      ArtifactWhen   `json:"when"`
-	Type      string         `json:"artifact_type"`
-	Format    ArtifactFormat `json:"artifact_format"`
-	ExpireIn  string         `json:"expire_in"`
+	Name      string          `json:"name"`
+	Untracked bool            `json:"untracked"`
+	Paths     ArtifactPaths   `json:"paths"`
+	Exclude   ArtifactExclude `json:"exclude"`
+	When      ArtifactWhen    `json:"when"`
+	Type      string          `json:"artifact_type"`
+	Format    ArtifactFormat  `json:"artifact_format"`
+	ExpireIn  string          `json:"expire_in"`
 }
 
 type Artifacts []Artifact
