@@ -18,6 +18,7 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/custom/command"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/process"
 )
 
 type executorTestCase struct {
@@ -32,7 +33,7 @@ type executorTestCase struct {
 	adjustExecutor func(t *testing.T, e *executor)
 
 	assertBuild          func(t *testing.T, b *common.Build)
-	assertCommandFactory func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions)
+	assertCommandFactory func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions)
 	assertOutput         func(t *testing.T, output string)
 	expectedError        string
 }
@@ -147,7 +148,7 @@ func mockCommandFactory(t *testing.T, tt executorTestCase) func() {
 		Return(tt.commandErr)
 
 	oldFactory := commandFactory
-	commandFactory = func(ctx context.Context, executable string, args []string, options command.CreateOptions) command.Command {
+	commandFactory = func(ctx context.Context, executable string, args []string, options process.CommandOptions) command.Command {
 		if tt.assertCommandFactory != nil {
 			tt.assertCommandFactory(t, tt, ctx, executable, args, options)
 		}
@@ -197,7 +198,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				ConfigArgs: []string{"test"},
 			}),
 			commandErr: errors.New("test-error"),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 				assert.Equal(t, tt.config.Custom.ConfigArgs, args)
 			},
@@ -213,7 +214,7 @@ func TestExecutor_Prepare(t *testing.T) {
 			}),
 			commandStdoutContent: "abcd",
 			commandErr:           nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -228,7 +229,7 @@ func TestExecutor_Prepare(t *testing.T) {
 			}),
 			commandStdoutContent: "",
 			commandErr:           nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -246,7 +247,7 @@ func TestExecutor_Prepare(t *testing.T) {
 			}),
 			commandStdoutContent: `{"builds_dir":""}`,
 			commandErr:           nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -265,7 +266,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				}
 			}`,
 			commandErr: nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -283,7 +284,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				}
 			}`,
 			commandErr: nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -306,7 +307,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				}
 			}`,
 			commandErr: nil,
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -324,7 +325,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				PrepareExec: "echo",
 				PrepareArgs: []string{"test"},
 			}),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.PrepareExec, executable)
 				assert.Equal(t, tt.config.Custom.PrepareArgs, args)
 			},
@@ -339,7 +340,7 @@ func TestExecutor_Prepare(t *testing.T) {
 				PrepareArgs: []string{"test"},
 			}),
 			commandErr: errors.New("test-error"),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.PrepareExec, executable)
 				assert.Equal(t, tt.config.Custom.PrepareArgs, args)
 			},
@@ -402,7 +403,7 @@ func TestExecutor_Cleanup(t *testing.T) {
 				CleanupExec: "echo",
 				CleanupArgs: []string{"test"},
 			}),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.CleanupExec, executable)
 				assert.Equal(t, tt.config.Custom.CleanupArgs, args)
 			},
@@ -418,7 +419,7 @@ func TestExecutor_Cleanup(t *testing.T) {
 			commandStdoutContent: "some output message in commands output",
 			commandStderrContent: "some error message in commands output",
 			commandErr:           errors.New("test-error"),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.CleanupExec, executable)
 			},
 			assertOutput: func(t *testing.T, output string) {
@@ -463,7 +464,7 @@ func TestExecutor_Run(t *testing.T) {
 			config: getRunnerConfig(&common.CustomConfig{
 				RunExec: "bash",
 			}),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.RunExec, executable)
 			},
 		},
@@ -473,7 +474,7 @@ func TestExecutor_Run(t *testing.T) {
 				CleanupExec: "unknown",
 			}),
 			commandErr: errors.New("test-error"),
-			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+			assertCommandFactory: func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 				assert.Equal(t, tt.config.Custom.RunExec, executable)
 			},
 			expectedError: "test-error",
@@ -520,8 +521,8 @@ func TestExecutor_Env(t *testing.T) {
 		CleanupExec: "bash",
 	})
 
-	assertCommandFactory := func(expectedImageName string) func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
-		return func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options command.CreateOptions) {
+	assertCommandFactory := func(expectedImageName string) func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
+		return func(t *testing.T, tt executorTestCase, ctx context.Context, executable string, args []string, options process.CommandOptions) {
 			for _, env := range options.Env {
 				pair := strings.Split(env, "=")
 				if pair[0] == ciJobImageEnv {
