@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"context"
 	"crypto/md5"
+	"errors"
 	"fmt"
 	"io"
 	"net/url"
@@ -77,6 +78,76 @@ func TestDockerCommandSuccessRun(t *testing.T) {
 	assert.NoError(t, err)
 }
 
+func TestDockerCommandMultistepBuild(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	tests := map[string]struct {
+		buildGetter    func() (common.JobResponse, error)
+		expectedOutput []string
+		unwantedOutput []string
+		errExpected    bool
+	}{
+		"Successful build with release and after_script step": {
+			buildGetter: common.GetRemoteSuccessfulMultistepBuild,
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo Release",
+				"echo After Script",
+			},
+			errExpected: false,
+		},
+		"Failure on script step. Release is skipped. After script runs.": {
+			buildGetter: func() (common.JobResponse, error) {
+				return common.GetRemoteFailingMultistepBuild(common.StepNameScript)
+			},
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo After Script",
+			},
+			unwantedOutput: []string{
+				"echo Release",
+			},
+			errExpected: true,
+		},
+		"Failure on release step. After script runs.": {
+			buildGetter: func() (common.JobResponse, error) {
+				return common.GetRemoteFailingMultistepBuild("release")
+			},
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo Release",
+				"echo After Script",
+			},
+			errExpected: true,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			build := getBuildForOS(t, tt.buildGetter)
+
+			var buf bytes.Buffer
+			err := build.Run(&common.Config{}, &common.Trace{Writer: &buf})
+
+			out := buf.String()
+			for _, output := range tt.expectedOutput {
+				assert.Contains(t, out, output)
+			}
+
+			for _, output := range tt.unwantedOutput {
+				assert.NotContains(t, out, output)
+			}
+
+			if tt.errExpected {
+				var buildErr *common.BuildError
+				assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+				return
+			}
+			assert.NoError(t, err)
+		})
+	}
+}
+
 func getBuildForOS(t *testing.T, getJobResp func() (common.JobResponse, error)) common.Build {
 	executor := "docker"
 	image := common.TestAlpineImage
@@ -149,44 +220,44 @@ func TestDockerCommandUsingCustomClonePath(t *testing.T) {
 		return
 	}
 
-	jobResponse, err := common.GetRemoteBuildResponse(
-		"ls -al $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo")
-	require.NoError(t, err)
+	remoteBuild := func() (common.JobResponse, error) {
+		cmd := "ls -al $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo"
+		if runtime.GOOS == "windows" {
+			cmd = "Get-Item -Path $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo"
+		}
+
+		return common.GetRemoteBuildResponse(cmd)
+	}
 
 	tests := map[string]struct {
-		clonePath         string
-		expectedErrorType interface{}
+		clonePath   string
+		expectedErr bool
 	}{
 		"uses custom clone path": {
-			clonePath:         "$CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo",
-			expectedErrorType: nil,
+			clonePath:   "$CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo",
+			expectedErr: false,
 		},
 		"path has to be within CI_BUILDS_DIR": {
-			clonePath:         "/unknown/go/src/gitlab.com/gitlab-org/repo",
-			expectedErrorType: &common.BuildError{},
+			clonePath:   "/unknown/go/src/gitlab.com/gitlab-org/repo",
+			expectedErr: true,
 		},
 	}
 
 	for name, test := range tests {
 		t.Run(name, func(t *testing.T) {
-			build := &common.Build{
-				JobResponse: jobResponse,
-				Runner: &common.RunnerConfig{
-					RunnerSettings: common.RunnerSettings{
-						Executor: "docker",
-						Docker: &common.DockerConfig{
-							Image:      common.TestAlpineImage,
-							PullPolicy: common.PullPolicyIfNotPresent,
-						},
-						Environment: []string{
-							"GIT_CLONE_PATH=" + test.clonePath,
-						},
-					},
-				},
+			build := getBuildForOS(t, remoteBuild)
+			build.Runner.Environment = []string{
+				"GIT_CLONE_PATH=" + test.clonePath,
 			}
 
-			err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-			assert.IsType(t, test.expectedErrorType, err)
+			err := buildtest.RunBuild(t, &build)
+			if test.expectedErr {
+				var buildErr *common.BuildError
+				assert.True(t, errors.As(err, &buildErr), "expected err %T, but got %T", buildErr, err)
+				return
+			}
+
+			assert.NoError(t, err)
 		})
 	}
 }
@@ -535,8 +606,9 @@ func TestDockerCommandOutput(t *testing.T) {
 	err = build.Run(&common.Config{}, &common.Trace{Writer: &buffer})
 	assert.NoError(t, err)
 
-	re, err := regexp.Compile("(?m)^Initialized empty Git repository in /builds/gitlab-org/ci-cd/tests/gitlab-test/.git/")
-	assert.NoError(t, err)
+	re, err :=
+		regexp.Compile("(?m)^Initialized empty Git repository in /builds/gitlab-org/ci-cd/tests/gitlab-test/.git/")
+	require.NoError(t, err)
 	assert.Regexp(t, re, buffer.String())
 }
 
@@ -1185,7 +1257,7 @@ func TestDockerCommandRunAttempts(t *testing.T) {
 		assertFailedToInspectContainer(t, trace, &attempts)
 	}
 
-	assert.Equal(t, executorStageAttempts, attempts, "The %s stage should be retried at least once", common.BuildStageUserScript)
+	assert.Equal(t, executorStageAttempts, attempts, "The %s stage should be retried at least once", "step_script")
 	<-runFinished
 }
 
