@@ -10,6 +10,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/docker/docker/api/types"
 	"github.com/jpillora/backoff"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
@@ -19,12 +20,14 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/kubernetes"
 	_ "k8s.io/client-go/plugin/pkg/client/auth" // Register all available authentication methods
+	restclient "k8s.io/client-go/rest"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/helperimage"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/services"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/dns"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker/auth"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/retry"
 	"gitlab.com/gitlab-org/gitlab-runner/session/proxy"
@@ -36,6 +39,8 @@ const (
 	helperContainerName = "helper"
 
 	detectShellScriptName = "detect_shell_script"
+
+	waitLogFileTimeout = time.Minute
 )
 
 var (
@@ -52,9 +57,7 @@ var (
 		ShowHostname: true,
 	}
 
-	detectShellScript = shells.BashTrapShellScript + shells.BashDetectShellScript
-
-	newLogProcessor = newKubernetesLogProcessor
+	detectShellScript = shells.BashDetectShellScript
 )
 
 type commandTerminatedError struct {
@@ -88,6 +91,7 @@ type executor struct {
 	executors.AbstractExecutor
 
 	kubeClient  *kubernetes.Clientset
+	kubeConfig  *restclient.Config
 	pod         *api.Pod
 	configMap   *api.ConfigMap
 	credentials *api.Secret
@@ -107,6 +111,8 @@ type executor struct {
 
 	featureChecker featureChecker
 
+	newLogProcessor func() logProcessor
+
 	remoteProcessTerminated chan shells.TrapCommandExitStatus
 }
 
@@ -159,7 +165,7 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) (err error) {
 		return fmt.Errorf("kubernetes doesn't support shells that require script file")
 	}
 
-	if err = s.prepareOverwrites(options.Build.Variables); err != nil {
+	if err = s.prepareOverwrites(options.Build.GetAllVariables()); err != nil {
 		return fmt.Errorf("couldn't prepare overwrites: %w", err)
 	}
 
@@ -177,8 +183,14 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) (err error) {
 		return fmt.Errorf("check defaults error: %w", err)
 	}
 
-	if s.kubeClient, err = getKubeClient(options.Config.Kubernetes, s.configurationOverwrites); err != nil {
-		return fmt.Errorf("error connecting to Kubernetes: %w", err)
+	s.kubeConfig, err = getKubeClientConfig(s.Config.Kubernetes, s.configurationOverwrites)
+	if err != nil {
+		return fmt.Errorf("getting Kubernetes config: %w", err)
+	}
+
+	s.kubeClient, err = kubernetes.NewForConfig(s.kubeConfig)
+	if err != nil {
+		return fmt.Errorf("connecting to Kubernetes: %w", err)
 	}
 
 	s.featureChecker = &kubeClientFeatureChecker{kubeClient: s.kubeClient}
@@ -257,13 +269,21 @@ func (s *executor) runWithAttach(cmd common.ExecutorCommand) error {
 	// Translates to roughly "sh /detect/shell/path.sh /stage/script/path.sh"
 	// which when the detect shell exits becomes something like "bash /stage/script/path.sh".
 	// This works unlike "gitlab-runner-build" since the detect shell passes arguments with "$@"
-	containerCommand := []string{"sh", s.scriptPath(detectShellScriptName), s.scriptPath(cmd.Stage)}
+	containerCommand := []string{
+		"sh",
+		s.scriptPath(detectShellScriptName),
+		s.buildCommandForStage(cmd.Stage),
+	}
 	if cmd.Predefined {
 		containerName = helperContainerName
 		// We use redirection here since the "gitlab-runner-build" helper doesn't pass input args
 		// to the shell it executes, so we technically pass the script to the stdin of the underlying shell
 		// translates roughly to "gitlab-runner-build <<< /stage/script/path.sh"
-		containerCommand = append(s.helperImageInfo.Cmd, "<<<", s.scriptPath(cmd.Stage))
+		containerCommand = append(
+			s.helperImageInfo.Cmd,
+			"<<<",
+			s.buildCommandForStage(cmd.Stage),
+		)
 	}
 
 	s.Debugln(fmt.Sprintf(
@@ -324,27 +344,22 @@ func (s *executor) ensurePodsConfigured(ctx context.Context) error {
 	return nil
 }
 
-func (s *executor) processLogs(ctx context.Context) {
-	processor := newLogProcessor(
-		s.kubeClient,
-		backoff.Backoff{Min: time.Second, Max: 30 * time.Second},
-		&s.BuildLogger,
-		kubernetesLogProcessorPodConfig{
-			namespace:  s.pod.Namespace,
-			pod:        s.pod.Name,
-			containers: []string{helperContainerName, buildContainerName},
-		},
-	)
+func (s *executor) buildCommandForStage(stage common.BuildStage) string {
+	return fmt.Sprintf("%s 2>&1 | tee -a %s", s.scriptPath(stage), s.logFile())
+}
 
-	logs := processor.Listen(ctx)
-	for line := range logs {
+func (s *executor) processLogs(ctx context.Context) {
+	processor := s.newLogProcessor()
+	logsCh := processor.Process(ctx)
+
+	for line := range logsCh {
 		var status shells.TrapCommandExitStatus
 		if status.TryUnmarshal(line) {
 			s.remoteProcessTerminated <- status
 			continue
 		}
 
-		_, err := s.Trace.Write([]byte(line + "\n"))
+		_, err := s.Trace.Write(append([]byte(line), '\n'))
 		if err != nil {
 			s.Warningln(fmt.Sprintf("Error writing log line to trace: %v", err))
 		}
@@ -354,9 +369,6 @@ func (s *executor) processLogs(ctx context.Context) {
 func (s *executor) setupScriptsConfigMap() error {
 	s.Debugln("Setting up scripts config map")
 
-	scripts := map[string]string{}
-	scripts[detectShellScriptName] = detectShellScript
-
 	// After issue https://gitlab.com/gitlab-org/gitlab-runner/issues/10342 is resolved and the legacy execution mode is removed
 	// we can remove the manual construction of trapShell and just use "bash+trap"
 	// in the exec options
@@ -365,13 +377,10 @@ func (s *executor) setupScriptsConfigMap() error {
 		return fmt.Errorf("kubernetes executor incorrect shell type")
 	}
 
-	trapShell := &shells.BashTrapShell{BashShell: bashShell}
-	for _, stage := range common.BuildStages {
-		script, err := trapShell.GenerateScript(stage, *s.Shell())
-		if err != nil {
-			return fmt.Errorf("generating trap shell script: %w", err)
-		}
-		scripts[string(stage)] = script
+	trapShell := &shells.BashTrapShell{BashShell: bashShell, LogFile: s.logFile()}
+	scripts, err := s.generateScripts(trapShell)
+	if err != nil {
+		return err
 	}
 
 	configMap := &api.ConfigMap{
@@ -382,7 +391,6 @@ func (s *executor) setupScriptsConfigMap() error {
 		Data: scripts,
 	}
 
-	var err error
 	s.configMap, err = s.kubeClient.CoreV1().ConfigMaps(s.configurationOverwrites.namespace).Create(configMap)
 	if err != nil {
 		return fmt.Errorf("generating scripts config map: %w", err)
@@ -391,6 +399,24 @@ func (s *executor) setupScriptsConfigMap() error {
 	return nil
 }
 
+func (s *executor) generateScripts(shell common.Shell) (map[string]string, error) {
+	scripts := map[string]string{}
+	scripts[detectShellScriptName] = detectShellScript
+
+	for _, stage := range s.Build.BuildStages() {
+		script, err := shell.GenerateScript(stage, *s.Shell())
+		if errors.Is(err, common.ErrSkipBuildStage) {
+			continue
+		} else if err != nil {
+			return nil, fmt.Errorf("generating trap shell script: %w", err)
+		}
+
+		scripts[string(stage)] = script
+	}
+
+	return scripts, nil
+}
+
 func (s *executor) Cleanup() {
 	s.cleanupResources()
 	s.cleanupServices()
@@ -509,6 +535,14 @@ func (s *executor) getCommandAndArgs(imageDefinition common.Image, command ...st
 	return command, args
 }
 
+func (s *executor) logFile() string {
+	return path.Join(s.logsDir(), "output.log")
+}
+
+func (s *executor) logsDir() string {
+	return path.Join(s.Build.TmpProjectDir(), "logs")
+}
+
 func (s *executor) scriptsDir() string {
 	return path.Join(s.Build.TmpProjectDir(), "scripts")
 }
@@ -531,6 +565,11 @@ func (s *executor) getVolumeMounts() []api.VolumeMount {
 			Name:      "scripts",
 			MountPath: s.scriptsDir(),
 		})
+
+		mounts = append(mounts, api.VolumeMount{
+			Name:      "logs",
+			MountPath: s.logsDir(),
+		})
 	}
 
 	mounts = append(mounts, s.getVolumeMountsForConfig()...)
@@ -612,6 +651,13 @@ func (s *executor) getVolumes() []api.Volume {
 		},
 	})
 
+	volumes = append(volumes, api.Volume{
+		Name: "logs",
+		VolumeSource: api.VolumeSource{
+			EmptyDir: &api.EmptyDirVolumeSource{},
+		},
+	})
+
 	return volumes
 }
 
@@ -697,31 +743,21 @@ func (s *executor) getVolumesForConfig() []api.Volume {
 	return volumes
 }
 
-type dockerConfigEntry struct {
-	Username, Password string
-}
-
 func (s *executor) setupCredentials() error {
 	s.Debugln("Setting up secrets")
 
-	authConfigs := make(map[string]dockerConfigEntry)
-
-	for _, credentials := range s.Build.Credentials {
-		if credentials.Type != "registry" {
-			continue
-		}
-
-		authConfigs[credentials.URL] = dockerConfigEntry{
-			Username: credentials.Username,
-			Password: credentials.Password,
-		}
-	}
+	authConfigs := auth.ResolveConfigs(s.Build.GetDockerAuthConfig(), s.Shell().User, s.Build.Credentials)
 
 	if len(authConfigs) == 0 {
 		return nil
 	}
 
-	dockerCfgContent, err := json.Marshal(authConfigs)
+	dockerCfgs := make(map[string]types.AuthConfig)
+	for registry, registryInfo := range authConfigs {
+		dockerCfgs[registry] = registryInfo.AuthConfig
+	}
+
+	dockerCfgContent, err := json.Marshal(dockerCfgs)
 	if err != nil {
 		return err
 	}
@@ -761,12 +797,13 @@ func (e *invalidHostAliasDNSError) Is(err error) bool {
 
 func (s *executor) prepareHostAlias() (*api.HostAlias, error) {
 	supportsHostAliases, err := s.featureChecker.IsHostAliasSupported()
-	if errors.Is(err, &badVersionError{}) {
+	switch {
+	case errors.Is(err, &badVersionError{}):
 		s.Warningln("Checking for host alias support. Host aliases will be disabled.", err)
 		return nil, nil
-	} else if err != nil {
+	case err != nil:
 		return nil, err
-	} else if !supportsHostAliases {
+	case !supportsHostAliases:
 		return nil, nil
 	}
 
@@ -1033,25 +1070,19 @@ func (s *executor) runInContainer(name string, command []string) <-chan error {
 	go func() {
 		defer close(errCh)
 
-		config, err := getKubeClientConfig(s.Config.Kubernetes, s.configurationOverwrites)
-		if err != nil {
-			errCh <- err
-			return
-		}
-
 		attach := AttachOptions{
 			PodName:       s.pod.Name,
 			Namespace:     s.pod.Namespace,
 			ContainerName: name,
 			Command:       command,
 
-			Config:   config,
+			Config:   s.kubeConfig,
 			Client:   s.kubeClient,
 			Executor: &DefaultRemoteExecutor{},
 		}
 
 		retryable := retry.New(retry.WithBuildLog(&attach, &s.BuildLogger))
-		err = retryable.Run()
+		err := retryable.Run()
 		if err != nil {
 			errCh <- err
 		}
@@ -1085,13 +1116,6 @@ func (s *executor) runInContainerWithExecLegacy(ctx context.Context, name string
 			return
 		}
 
-		config, err := getKubeClientConfig(s.Config.Kubernetes, s.configurationOverwrites)
-
-		if err != nil {
-			errCh <- err
-			return
-		}
-
 		exec := ExecOptions{
 			PodName:       s.pod.Name,
 			Namespace:     s.pod.Namespace,
@@ -1101,7 +1125,7 @@ func (s *executor) runInContainerWithExecLegacy(ctx context.Context, name string
 			Out:           s.Trace,
 			Err:           s.Trace,
 			Stdin:         true,
-			Config:        config,
+			Config:        s.kubeConfig,
 			Client:        s.kubeClient,
 			Executor:      &DefaultRemoteExecutor{},
 		}
@@ -1168,7 +1192,7 @@ func (s *executor) checkDefaults() error {
 	return nil
 }
 
-func createFn() common.Executor {
+func newExecutor() *executor {
 	helperImageInfo, err := helperimage.Get(common.REVISION, helperimage.Config{
 		OSType:       helperimage.OSTypeLinux,
 		Architecture: "amd64",
@@ -1177,13 +1201,31 @@ func createFn() common.Executor {
 		logrus.WithError(err).Fatal("Failed to set up helper image for kubernetes executor")
 	}
 
-	return &executor{
+	e := &executor{
 		AbstractExecutor: executors.AbstractExecutor{
 			ExecutorOptions: executorOptions,
 		},
 		helperImageInfo:         helperImageInfo,
 		remoteProcessTerminated: make(chan shells.TrapCommandExitStatus),
 	}
+
+	e.newLogProcessor = func() logProcessor {
+		return newKubernetesLogProcessor(
+			e.kubeClient,
+			e.kubeConfig,
+			&backoff.Backoff{Min: time.Second, Max: 30 * time.Second},
+			e.Build.Log(),
+			kubernetesLogProcessorPodConfig{
+				namespace:          e.pod.Namespace,
+				pod:                e.pod.Name,
+				container:          helperContainerName,
+				logPath:            e.logFile(),
+				waitLogFileTimeout: waitLogFileTimeout,
+			},
+		)
+	}
+
+	return e
 }
 
 func featuresFn(features *common.FeaturesInfo) {
@@ -1199,7 +1241,9 @@ func featuresFn(features *common.FeaturesInfo) {
 
 func init() {
 	common.RegisterExecutorProvider("kubernetes", executors.DefaultExecutorProvider{
-		Creator:          createFn,
+		Creator: func() common.Executor {
+			return newExecutor()
+		},
 		FeaturesUpdater:  featuresFn,
 		DefaultShellName: executorOptions.Shell.Shell,
 	})
