@@ -18,7 +18,6 @@ import (
 	"time"
 
 	"github.com/gorilla/websocket"
-	"github.com/jpillora/backoff"
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
@@ -47,6 +46,7 @@ type featureFlagTest func(t *testing.T, flagName string, flagValue bool)
 func TestRunTestsWithFeatureFlag(t *testing.T) {
 	tests := map[string]featureFlagTest{
 		"testKubernetesSuccessRun":              testKubernetesSuccessRunFeatureFlag,
+		"testKubernetesMultistepRun":            testKubernetesMultistepRunFeatureFlag,
 		"testKubernetesTimeoutRun":              testKubernetesTimeoutRunFeatureFlag,
 		"testKubernetesBuildFail":               testKubernetesBuildFailFeatureFlag,
 		"testKubernetesBuildAbort":              testKubernetesBuildAbortFeatureFlag,
@@ -105,6 +105,96 @@ func testKubernetesSuccessRunFeatureFlag(t *testing.T, featureFlagName string, f
 	assert.NoError(t, err)
 }
 
+func testKubernetesMultistepRunFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+	helpers.SkipIntegrationTests(t, "kubectl", "cluster-info")
+
+	successfulBuild, err := common.GetRemoteSuccessfulMultistepBuild()
+	require.NoError(t, err)
+
+	failingScriptBuild, err := common.GetRemoteFailingMultistepBuild(common.StepNameScript)
+	require.NoError(t, err)
+
+	failingReleaseBuild, err := common.GetRemoteFailingMultistepBuild("release")
+	require.NoError(t, err)
+
+	successfulBuild.Image.Name = common.TestDockerGitImage
+	failingScriptBuild.Image.Name = common.TestDockerGitImage
+	failingReleaseBuild.Image.Name = common.TestDockerGitImage
+
+	tests := map[string]struct {
+		jobResponse    common.JobResponse
+		expectedOutput []string
+		unwantedOutput []string
+		errExpected    bool
+	}{
+		"Successful build with release and after_script step": {
+			jobResponse: successfulBuild,
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo Release",
+				"echo After Script",
+			},
+			errExpected: false,
+		},
+		"Failure on script step. Release is skipped. After script runs.": {
+			jobResponse: failingScriptBuild,
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo After Script",
+			},
+			unwantedOutput: []string{
+				"echo Release",
+			},
+			errExpected: true,
+		},
+		"Failure on release step. After script runs.": {
+			jobResponse: failingReleaseBuild,
+			expectedOutput: []string{
+				"echo Hello World",
+				"echo Release",
+				"echo After Script",
+			},
+			errExpected: true,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			build := &common.Build{
+				JobResponse: tt.jobResponse,
+				Runner: &common.RunnerConfig{
+					RunnerSettings: common.RunnerSettings{
+						Executor: "kubernetes",
+						Kubernetes: &common.KubernetesConfig{
+							PullPolicy: common.PullPolicyIfNotPresent,
+						},
+					},
+				},
+			}
+			setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
+
+			var buf bytes.Buffer
+			err := build.Run(&common.Config{}, &common.Trace{Writer: &buf})
+
+			out := buf.String()
+			for _, output := range tt.expectedOutput {
+				assert.Contains(t, out, output)
+			}
+
+			for _, output := range tt.unwantedOutput {
+				assert.NotContains(t, out, output)
+			}
+
+			if tt.errExpected {
+				var buildErr *common.BuildError
+				assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+				return
+			}
+			assert.NoError(t, err)
+		})
+	}
+}
+
 func testKubernetesTimeoutRunFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	if helpers.SkipIntegrationTests(t, "kubectl", "cluster-info") {
 		return
@@ -333,6 +423,7 @@ func testVolumeMountsFeatureFlag(t *testing.T, featureFlagName string, featureFl
 					Config:          tt.RunnerConfig,
 				},
 			}
+
 			setBuildFeatureFlag(e.Build, featureFlagName, featureFlagValue)
 
 			mounts := e.getVolumeMounts()
@@ -432,6 +523,7 @@ func testVolumesFeatureFlag(t *testing.T, featureFlagName string, featureFlagVal
 				},
 				configMap: fakeConfigMap(),
 			}
+
 			setBuildFeatureFlag(e.Build, featureFlagName, featureFlagValue)
 
 			volumes := e.getVolumes()
@@ -545,16 +637,16 @@ func testKubernetesCustomClonePathFeatureFlag(t *testing.T, featureFlagName stri
 	require.NoError(t, err)
 
 	tests := map[string]struct {
-		clonePath         string
-		expectedErrorType interface{}
+		clonePath   string
+		expectedErr bool
 	}{
 		"uses custom clone path": {
-			clonePath:         "$CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo",
-			expectedErrorType: nil,
+			clonePath:   "$CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo",
+			expectedErr: false,
 		},
 		"path has to be within CI_BUILDS_DIR": {
-			clonePath:         "/unknown/go/src/gitlab.com/gitlab-org/repo",
-			expectedErrorType: &common.BuildError{},
+			clonePath:   "/unknown/go/src/gitlab.com/gitlab-org/repo",
+			expectedErr: true,
 		},
 	}
 
@@ -578,7 +670,13 @@ func testKubernetesCustomClonePathFeatureFlag(t *testing.T, featureFlagName stri
 			setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
 
 			err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-			assert.IsType(t, test.expectedErrorType, err)
+			if test.expectedErr {
+				var buildErr *common.BuildError
+				assert.True(t, errors.As(err, &buildErr), "expected err %T, but got %T", buildErr, err)
+				return
+			}
+
+			assert.NoError(t, err)
 		})
 	}
 }
@@ -631,7 +729,8 @@ func testKubernetesMissingImageFeatureFlag(t *testing.T, featureFlagName string,
 
 	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	assert.IsType(t, err, &common.BuildError{})
+	var buildErr *common.BuildError
+	assert.True(t, errors.As(err, &buildErr), "expected err %T, but got %T", buildErr, err)
 	assert.Contains(t, err.Error(), "image pull failed")
 }
 
@@ -656,7 +755,8 @@ func testKubernetesMissingTagFeatureFlag(t *testing.T, featureFlagName string, f
 
 	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	assert.IsType(t, err, &common.BuildError{})
+	var buildErr *common.BuildError
+	assert.True(t, errors.As(err, &buildErr), "expected err %T, but got %T", buildErr, err)
 	assert.Contains(t, err.Error(), "image pull failed")
 }
 
@@ -737,7 +837,9 @@ func testInteractiveTerminalFeatureFlag(t *testing.T, featureFlagName string, fe
 		return
 	}
 
-	client, err := getKubeClient(&common.KubernetesConfig{}, &overwrites{})
+	config, err := getKubeClientConfig(new(common.KubernetesConfig), new(overwrites))
+	require.NoError(t, err)
+	client, err := kubernetes.NewForConfig(config)
 	require.NoError(t, err)
 	secrets, err := client.CoreV1().Secrets("default").List(metav1.ListOptions{})
 	require.NoError(t, err)
@@ -968,9 +1070,10 @@ func TestCleanup(t *testing.T) {
 				configMap:   test.ConfigMap,
 			}
 			ex.configurationOverwrites = &overwrites{namespace: "test-ns"}
+
 			errored := false
 			buildTrace := FakeBuildTrace{
-				testWriter{
+				testWriter: testWriter{
 					call: func(b []byte) (int, error) {
 						if !errored {
 							if s := string(b); strings.Contains(s, "Error cleaning up") {
@@ -1311,8 +1414,9 @@ func TestPrepare(t *testing.T) {
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
 					Kubernetes: &common.KubernetesConfig{
-						Namespace: "namespace",
-						Host:      "test-server",
+						Namespace:                 "namespace",
+						Host:                      "test-server",
+						NamespaceOverwriteAllowed: "^namespace-[0-9]$",
 					},
 				},
 			},
@@ -1325,7 +1429,7 @@ func TestPrepare(t *testing.T) {
 						Name: "test-image",
 					},
 					Variables: []common.JobVariable{
-						{Key: NamespaceOverwriteVariableName, Value: "namespace"},
+						{Key: NamespaceOverwriteVariableName, Value: "namespace-$CI_CONCURRENT_ID"},
 					},
 				},
 				Runner: &common.RunnerConfig{},
@@ -1336,7 +1440,7 @@ func TestPrepare(t *testing.T) {
 						Name: "test-image",
 					},
 				},
-				configurationOverwrites: &overwrites{namespace: "namespace"},
+				configurationOverwrites: &overwrites{namespace: "namespace-0"},
 				serviceLimits:           api.ResourceList{},
 				buildLimits:             api.ResourceList{},
 				helperLimits:            api.ResourceList{},
@@ -1533,6 +1637,7 @@ func TestPrepare(t *testing.T) {
 			// It currently contains some moving parts that are failing, meaning
 			// we'll need to mock _something_
 			e.kubeClient = nil
+			e.kubeConfig = nil
 			e.featureChecker = nil
 			assert.Equal(t, test.Expected, e)
 		})
@@ -2640,6 +2745,7 @@ func TestSetupBuildPod(t *testing.T) {
 
 			mockFc := &mockFeatureChecker{}
 			mockFc.On("IsHostAliasSupported").Return(true, nil)
+
 			ex := executor{
 				kubeClient: testKubernetesClient(version, fake.CreateHTTPClient(rt.RoundTrip)),
 				options:    options,
@@ -2684,54 +2790,41 @@ func TestSetupBuildPod(t *testing.T) {
 func TestProcessLogs(t *testing.T) {
 	mockTrace := &common.MockJobTrace{}
 	defer mockTrace.AssertExpectations(t)
+	mockTrace.On("Write", []byte("line\n")).Return(0, nil).Once()
 
-	mockTrace.On("Write", mock.MatchedBy(func(in []byte) bool {
-		return bytes.Equal(in, []byte("line\n"))
-	})).Return(0, nil).Once()
-
-	oldLogProcessor := newLogProcessor
-	defer func() {
-		newLogProcessor = oldLogProcessor
-	}()
-
-	mockLogProcessor := &mockLogProcessor{}
+	mockLogProcessor := new(mockLogProcessor)
 	defer mockLogProcessor.AssertExpectations(t)
 
-	logsCh := make(chan string)
-	go func() {
-		logsCh <- "line"
-
-		exitCode := 1
-		script := "script"
-		status := shells.TrapCommandExitStatus{
-			CommandExitCode: &exitCode,
-			Script:          &script,
-		}
-
-		b, _ := json.Marshal(status)
-		logsCh <- string(b)
-		close(logsCh)
-	}()
-
-	var returnCh <-chan string = logsCh
-	mockLogProcessor.On("Listen", mock.Anything).Return(returnCh).Once()
-
-	newLogProcessor = func(client *kubernetes.Clientset, backoff backoff.Backoff, logger *common.BuildLogger, podCfg kubernetesLogProcessorPodConfig) logProcessor {
-		return mockLogProcessor
+	ch := make(chan string, 2)
+	ch <- "line"
+	exitCode := 1
+	script := "script"
+	status := shells.TrapCommandExitStatus{
+		CommandExitCode: &exitCode,
+		Script:          &script,
 	}
 
-	e := executor{}
-	e.remoteProcessTerminated = make(chan shells.TrapCommandExitStatus)
+	b, err := json.Marshal(status)
+	require.NoError(t, err)
+	ch <- string(b)
+	mockLogProcessor.On("Process", mock.Anything).
+		Return((<-chan string)(ch)).
+		Once()
+
+	e := newExecutor()
 	e.Trace = mockTrace
 	e.pod = &api.Pod{}
 	e.pod.Name = "pod_name"
 	e.pod.Namespace = "namespace"
+	e.newLogProcessor = func() logProcessor {
+		return mockLogProcessor
+	}
 
 	go e.processLogs(context.Background())
 
 	exitStatus := <-e.remoteProcessTerminated
-	assert.Equal(t, 1, *exitStatus.CommandExitCode)
-	assert.Equal(t, "script", *exitStatus.Script)
+	assert.Equal(t, exitCode, *exitStatus.CommandExitCode)
+	assert.Equal(t, script, *exitStatus.Script)
 }
 
 func TestRunAttachCheckPodStatus(t *testing.T) {
@@ -2879,6 +2972,7 @@ func TestLimits(t *testing.T) {
 	tests := []struct {
 		CPU, Memory string
 		Expected    api.ResourceList
+		ExpectedErr error
 	}{
 		{
 			CPU:    "100m",
@@ -2887,35 +2981,49 @@ func TestLimits(t *testing.T) {
 				api.ResourceCPU:    resource.MustParse("100m"),
 				api.ResourceMemory: resource.MustParse("100Mi"),
 			},
+			ExpectedErr: nil,
 		},
 		{
 			CPU: "100m",
 			Expected: api.ResourceList{
 				api.ResourceCPU: resource.MustParse("100m"),
 			},
+			ExpectedErr: nil,
 		},
 		{
 			Memory: "100Mi",
 			Expected: api.ResourceList{
 				api.ResourceMemory: resource.MustParse("100Mi"),
 			},
+			ExpectedErr: nil,
 		},
 		{
-			CPU:      "100j",
-			Expected: api.ResourceList{},
+			CPU:         "100j",
+			Expected:    api.ResourceList{},
+			ExpectedErr: resource.ErrFormatWrong,
 		},
 		{
-			Memory:   "100j",
-			Expected: api.ResourceList{},
+			Memory:      "100j",
+			Expected:    api.ResourceList{},
+			ExpectedErr: resource.ErrFormatWrong,
 		},
 		{
-			Expected: api.ResourceList{},
+			Expected:    api.ResourceList{},
+			ExpectedErr: nil,
 		},
 	}
 
-	for _, test := range tests {
-		res, _ := limits(test.CPU, test.Memory)
-		assert.Equal(t, test.Expected, res)
+	for _, tc := range tests {
+		t.Run(fmt.Sprintf("CPU=%s/Memory=%s", tc.CPU, tc.Memory), func(t *testing.T) {
+			res, err := limits(tc.CPU, tc.Memory)
+			assert.True(t,
+				errors.Is(err, tc.ExpectedErr),
+				"expected err %T, but got %T",
+				tc.ExpectedErr,
+				err,
+			)
+			assert.Equal(t, tc.Expected, res)
+		})
 	}
 }
 
@@ -2948,6 +3056,61 @@ func setBuildFeatureFlag(build *common.Build, flag string, value bool) {
 	})
 }
 
+func TestNewLogStreamerStream(t *testing.T) {
+	abortErr := errors.New("abort")
+
+	pod := new(api.Pod)
+	pod.Namespace = "k8s_namespace"
+	pod.Name = "k8s_pod_name"
+
+	client := mockKubernetesClientWithHost("", "", nil)
+	output := new(bytes.Buffer)
+	offset := 15
+
+	e := newExecutor()
+	e.pod = pod
+	e.Build = &common.Build{
+		Runner: new(common.RunnerConfig),
+	}
+
+	remoteExecutor := new(MockRemoteExecutor)
+	urlMatcher := mock.MatchedBy(func(url *url.URL) bool {
+		query := url.Query()
+		assert.Equal(t, helperContainerName, query.Get("container"))
+		assert.Equal(t, "true", query.Get("stdout"))
+		assert.Equal(t, "true", query.Get("stderr"))
+		command := query["command"]
+		assert.Equal(t, []string{
+			"gitlab-runner-helper",
+			"read-logs",
+			"--path",
+			e.logFile(),
+			"--offset",
+			strconv.Itoa(offset),
+			"--wait-file-timeout",
+			waitLogFileTimeout.String(),
+		}, command)
+
+		return true
+	})
+	remoteExecutor.On("Execute", http.MethodPost, urlMatcher, mock.Anything, nil, output, output, false).Return(abortErr)
+
+	p, ok := e.newLogProcessor().(*kubernetesLogProcessor)
+	require.True(t, ok)
+	p.logsOffset = int64(offset)
+
+	s, ok := p.logStreamer.(*kubernetesLogStreamer)
+	require.True(t, ok)
+	s.client = client
+	s.executor = remoteExecutor
+
+	assert.Equal(t, pod.Name, s.pod)
+	assert.Equal(t, pod.Namespace, s.namespace)
+
+	err := s.Stream(context.Background(), int64(offset), output)
+	assert.True(t, errors.Is(err, abortErr))
+}
+
 type FakeReadCloser struct {
 	io.Reader
 }
@@ -2996,3 +3159,91 @@ func TestCommandTerminatedError_Is(t *testing.T) {
 		})
 	}
 }
+
+func TestGenerateScripts(t *testing.T) {
+	testErr := errors.New("testErr")
+
+	successfulResponse, err := common.GetRemoteSuccessfulMultistepBuild()
+	require.NoError(t, err)
+
+	e := &executor{
+		AbstractExecutor: executors.AbstractExecutor{
+			Build: &common.Build{
+				JobResponse: successfulResponse,
+			},
+		},
+	}
+	buildStages := e.Build.BuildStages()
+
+	setupMockShellGenerateScript := func(m *common.MockShell, stages []common.BuildStage) {
+		for _, s := range stages {
+			m.On("GenerateScript", s, e.ExecutorOptions.Shell).
+				Return("OK", nil).
+				Once()
+		}
+	}
+
+	setupScripts := func(stages []common.BuildStage) map[string]string {
+		scripts := map[string]string{}
+		scripts[detectShellScriptName] = detectShellScript
+
+		for _, s := range stages {
+			scripts[string(s)] = "OK"
+		}
+
+		return scripts
+	}
+
+	tests := map[string]struct {
+		setupMockShell  func() *common.MockShell
+		expectedScripts map[string]string
+		expectedErr     error
+	}{
+		"all stages OK": {
+			setupMockShell: func() *common.MockShell {
+				m := new(common.MockShell)
+				setupMockShellGenerateScript(m, buildStages)
+
+				return m
+			},
+			expectedScripts: setupScripts(buildStages),
+			expectedErr:     nil,
+		},
+		"stage returns skip build stage error": {
+			setupMockShell: func() *common.MockShell {
+				m := new(common.MockShell)
+				m.On("GenerateScript", buildStages[0], e.ExecutorOptions.Shell).
+					Return("", common.ErrSkipBuildStage).
+					Once()
+				setupMockShellGenerateScript(m, buildStages[1:])
+
+				return m
+			},
+			expectedScripts: setupScripts(buildStages[1:]),
+			expectedErr:     nil,
+		},
+		"stage returns error": {
+			setupMockShell: func() *common.MockShell {
+				m := new(common.MockShell)
+				m.On("GenerateScript", buildStages[0], e.ExecutorOptions.Shell).
+					Return("", testErr).
+					Once()
+
+				return m
+			},
+			expectedScripts: nil,
+			expectedErr:     testErr,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			m := tt.setupMockShell()
+			defer m.AssertExpectations(t)
+
+			scripts, err := e.generateScripts(m)
+			assert.True(t, errors.Is(err, tt.expectedErr), "expected err %T, but got %T", tt.expectedErr, err)
+			assert.Equal(t, tt.expectedScripts, scripts)
+		})
+	}
+}
