@@ -88,15 +88,6 @@ func loadDefaultKubectlConfig() (*restclient.Config, error) {
 	return clientcmd.NewDefaultClientConfig(*config, &clientcmd.ConfigOverrides{}).ClientConfig()
 }
 
-func getKubeClient(config *common.KubernetesConfig, overwrites *overwrites) (*kubernetes.Clientset, error) {
-	restConfig, err := getKubeClientConfig(config, overwrites)
-	if err != nil {
-		return nil, err
-	}
-
-	return kubernetes.NewForConfig(restConfig)
-}
-
 func closeKubeClient(client *kubernetes.Clientset) bool {
 	if client == nil {
 		return false
@@ -217,17 +208,17 @@ func limits(cpu, memory string) (api.ResourceList, error) {
 			return q, nil
 		}
 		if q, err = resource.ParseQuantity(s); err != nil {
-			return q, fmt.Errorf("error parsing resource limit: %w", err)
+			return q, fmt.Errorf("parsing resource limit: %w", err)
 		}
 		return q, nil
 	}
 
 	if rCPU, err = parse(cpu); err != nil {
-		return api.ResourceList{}, nil
+		return api.ResourceList{}, err
 	}
 
 	if rMem, err = parse(memory); err != nil {
-		return api.ResourceList{}, nil
+		return api.ResourceList{}, err
 	}
 
 	l := make(api.ResourceList)
