@@ -9,9 +9,8 @@ import (
 	"os/exec"
 
 	"github.com/kr/pty"
-	"gitlab.com/gitlab-org/gitlab-terminal"
-
 	terminalsession "gitlab.com/gitlab-org/gitlab-runner/session/terminal"
+	terminal "gitlab.com/gitlab-org/gitlab-terminal"
 )
 
 type terminalConn struct {
