@@ -9,6 +9,8 @@ const (
 	NetworkPerBuild                      string = "FF_NETWORK_PER_BUILD"
 	UseLegacyKubernetesExecutionStrategy string = "FF_USE_LEGACY_KUBERNETES_EXECUTION_STRATEGY"
 	UseDirectDownload                    string = "FF_USE_DIRECT_DOWNLOAD"
+	SkipNoOpBuildStages                  string = "FF_SKIP_NOOP_BUILD_STAGES"
+	ShellExecutorUseLegacyProcessKill    string = "FF_SHELL_EXECUTOR_USE_LEGACY_PROCESS_KILL"
 )
 
 type FeatureFlag struct {
@@ -37,21 +39,35 @@ var flags = []FeatureFlag{
 		DefaultValue:    "false",
 		Deprecated:      false,
 		ToBeRemovedWith: "",
-		Description:     "Enables creation of a docker [network per build](../executors/docker.md#networking) with the docker executor",
+		Description:     "Enables creation of a Docker [network per build](../executors/docker.md#networking) with the `docker` executor",
 	},
 	{
 		Name:            UseLegacyKubernetesExecutionStrategy,
 		DefaultValue:    "true",
 		Deprecated:      false,
 		ToBeRemovedWith: "",
-		Description:     "When set to `false` disables execution of remote Kubernetes commands through `exec` in favor of `attach` to solve problems like [#4119](https://gitlab.com/gitlab-org/gitlab-runner/issues/4119)",
+		Description:     "When set to `false` disables execution of remote Kubernetes commands through `exec` in favor of `attach` to solve problems like [#4119](https://gitlab.com/gitlab-org/gitlab-runner/-/issues/4119)",
 	},
 	{
 		Name:            UseDirectDownload,
-		DefaultValue:    "false",
+		DefaultValue:    "true",
 		Deprecated:      false,
 		ToBeRemovedWith: "",
-		Description:     "When set to `true` Runner tries to direct-download all artifacts instead of proxying through GitLab. Enabling might result in a download failures due to problem validating TLS certificate of Object Storage if it is enabled by GitLab",
+		Description:     "When set to `true` Runner tries to direct-download all artifacts instead of proxying through GitLab on a first try. Enabling might result in a download failures due to problem validating TLS certificate of Object Storage if it is enabled by GitLab",
+	},
+	{
+		Name:            SkipNoOpBuildStages,
+		DefaultValue:    "true",
+		Deprecated:      false,
+		ToBeRemovedWith: "",
+		Description:     "When set to `false` all build stages are executed even if running them has no effect",
+	},
+	{
+		Name:            ShellExecutorUseLegacyProcessKill,
+		DefaultValue:    "false",
+		Deprecated:      true,
+		ToBeRemovedWith: "14.0",
+		Description:     "Use the old process termination that was used prior to GitLab 13.1 where only `SIGKILL` was sent",
 	},
 }
 
