@@ -25,6 +25,7 @@ func (b *AbstractShell) GetFeatures(features *common.FeaturesInfo) {
 	features.Refspecs = true
 	features.Masking = true
 	features.RawVariables = true
+	features.ArtifactsExclude = true
 }
 
 func (b *AbstractShell) writeCdBuildDir(w ShellWriter, info common.ShellScriptInfo) {
@@ -69,6 +70,8 @@ func (b *AbstractShell) guardRunnerCommand(w ShellWriter, runnerCommand string,
 }
 
 func (b *AbstractShell) cacheExtractor(w ShellWriter, info common.ShellScriptInfo) error {
+	skipRestoreCache := true
+
 	for _, cacheOptions := range info.Build.Cache {
 		// Create list of files to extract
 		var archiverArgs []string
@@ -85,6 +88,8 @@ func (b *AbstractShell) cacheExtractor(w ShellWriter, info common.ShellScriptInf
 			continue
 		}
 
+		skipRestoreCache = false
+
 		// Skip extraction if no cache is defined
 		cacheKey, cacheFile := b.cacheFile(info.Build, cacheOptions.Key)
 		if cacheKey == "" {
@@ -121,6 +126,10 @@ func (b *AbstractShell) cacheExtractor(w ShellWriter, info common.ShellScriptInf
 		})
 	}
 
+	if skipRestoreCache {
+		return common.ErrSkipBuildStage
+	}
+
 	return nil
 }
 
@@ -150,10 +159,10 @@ func (b *AbstractShell) jobArtifacts(info common.ShellScriptInfo) (otherJobs []c
 	return
 }
 
-func (b *AbstractShell) downloadAllArtifacts(w ShellWriter, info common.ShellScriptInfo) {
+func (b *AbstractShell) downloadAllArtifacts(w ShellWriter, info common.ShellScriptInfo) error {
 	otherJobs := b.jobArtifacts(info)
 	if len(otherJobs) == 0 {
-		return
+		return common.ErrSkipBuildStage
 	}
 
 	b.guardRunnerCommand(w, info.RunnerCommand, "Artifacts downloading", func() {
@@ -161,13 +170,15 @@ func (b *AbstractShell) downloadAllArtifacts(w ShellWriter, info common.ShellScr
 			b.downloadArtifacts(w, otherJob, info)
 		}
 	})
-}
 
-func (b *AbstractShell) writePrepareScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
 	return nil
 }
 
-func (b *AbstractShell) writeGetSourcesScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writePrepareScript(w ShellWriter, info common.ShellScriptInfo) error {
+	return nil
+}
+
+func (b *AbstractShell) writeGetSourcesScript(w ShellWriter, info common.ShellScriptInfo) error {
 	b.writeExports(w, info)
 
 	if !info.Build.IsSharedEnv() {
@@ -314,12 +325,14 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, build *common.Build,
 	w.Command("git", "remote", "set-url", "origin", build.GetRemoteURL())
 	w.EndIf()
 
-	fetchArgs := []string{"fetch", "origin", "--prune"}
+	fetchArgs := []string{"fetch", "origin"}
 	fetchArgs = append(fetchArgs, build.GitInfo.Refspecs...)
 	if depth > 0 {
 		fetchArgs = append(fetchArgs, "--depth", strconv.Itoa(depth))
 	}
 
+	fetchArgs = append(fetchArgs, build.GetGitFetchFlags()...)
+
 	w.Command("git", fetchArgs...)
 }
 
@@ -344,7 +357,7 @@ func (b *AbstractShell) writeCheckoutCmd(w ShellWriter, build *common.Build) {
 	}
 }
 
-func (b *AbstractShell) writeSubmoduleUpdateCmds(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writeSubmoduleUpdateCmds(w ShellWriter, info common.ShellScriptInfo) error {
 	build := info.Build
 
 	switch build.GetSubmoduleStrategy() {
@@ -399,7 +412,7 @@ func (b *AbstractShell) writeSubmoduleUpdateCmd(w ShellWriter, build *common.Bui
 	}
 }
 
-func (b *AbstractShell) writeRestoreCacheScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writeRestoreCacheScript(w ShellWriter, info common.ShellScriptInfo) error {
 	b.writeExports(w, info)
 	b.writeCdBuildDir(w, info)
 
@@ -407,13 +420,11 @@ func (b *AbstractShell) writeRestoreCacheScript(w ShellWriter, info common.Shell
 	return b.cacheExtractor(w, info)
 }
 
-func (b *AbstractShell) writeDownloadArtifactsScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writeDownloadArtifactsScript(w ShellWriter, info common.ShellScriptInfo) error {
 	b.writeExports(w, info)
 	b.writeCdBuildDir(w, info)
 
-	// Process all artifacts
-	b.downloadAllArtifacts(w, info)
-	return nil
+	return b.downloadAllArtifacts(w, info)
 }
 
 // Write the given string of commands using the provided ShellWriter object.
@@ -436,17 +447,17 @@ func (b *AbstractShell) writeCommands(w ShellWriter, commands ...string) {
 	}
 }
 
-func (b *AbstractShell) writeUserScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writeUserScript(w ShellWriter, info common.ShellScriptInfo, buildStage common.BuildStage) error {
 	var scriptStep *common.Step
 	for _, step := range info.Build.Steps {
-		if step.Name == common.StepNameScript {
+		if common.StepToBuildStage(step) == buildStage {
 			scriptStep = &step
 			break
 		}
 	}
 
 	if scriptStep == nil {
-		return nil
+		return common.ErrSkipBuildStage
 	}
 
 	b.writeExports(w, info)
@@ -466,7 +477,26 @@ func (b *AbstractShell) writeUserScript(w ShellWriter, info common.ShellScriptIn
 }
 
 func (b *AbstractShell) cacheArchiver(w ShellWriter, info common.ShellScriptInfo) error {
+	skipArchiveCache := true
+
 	for _, cacheOptions := range info.Build.Cache {
+		// Create list of files to archive
+		var archiverArgs []string
+		for _, path := range cacheOptions.Paths {
+			archiverArgs = append(archiverArgs, "--path", path)
+		}
+
+		if cacheOptions.Untracked {
+			archiverArgs = append(archiverArgs, "--untracked")
+		}
+
+		if len(archiverArgs) < 1 {
+			// Skip creating archive
+			continue
+		}
+
+		skipArchiveCache = false
+
 		// Skip archiving if no cache is defined
 		cacheKey, cacheFile := b.cacheFile(info.Build, cacheOptions.Key)
 		if cacheKey == "" {
@@ -487,20 +517,6 @@ func (b *AbstractShell) cacheArchiver(w ShellWriter, info common.ShellScriptInfo
 			"--timeout", strconv.Itoa(info.Build.GetCacheRequestTimeout()),
 		}
 
-		// Create list of files to archive
-		var archiverArgs []string
-		for _, path := range cacheOptions.Paths {
-			archiverArgs = append(archiverArgs, "--path", path)
-		}
-
-		if cacheOptions.Untracked {
-			archiverArgs = append(archiverArgs, "--untracked")
-		}
-
-		if len(archiverArgs) < 1 {
-			// Skip creating archive
-			continue
-		}
 		args = append(args, archiverArgs...)
 
 		// Generate cache upload address
@@ -519,10 +535,14 @@ func (b *AbstractShell) cacheArchiver(w ShellWriter, info common.ShellScriptInfo
 		})
 	}
 
+	if skipArchiveCache {
+		return common.ErrSkipBuildStage
+	}
+
 	return nil
 }
 
-func (b *AbstractShell) writeUploadArtifact(w ShellWriter, info common.ShellScriptInfo, artifact common.Artifact) {
+func (b *AbstractShell) writeUploadArtifact(w ShellWriter, info common.ShellScriptInfo, artifact common.Artifact) bool {
 	args := []string{
 		"artifacts-uploader",
 		"--url",
@@ -539,14 +559,20 @@ func (b *AbstractShell) writeUploadArtifact(w ShellWriter, info common.ShellScri
 		archiverArgs = append(archiverArgs, "--path", path)
 	}
 
+	// Create list of paths to be excluded from the archive
+	for _, path := range artifact.Exclude {
+		archiverArgs = append(archiverArgs, "--exclude", path)
+	}
+
 	if artifact.Untracked {
 		archiverArgs = append(archiverArgs, "--untracked")
 	}
 
 	if len(archiverArgs) < 1 {
 		// Skip creating archive
-		return
+		return false
 	}
+
 	args = append(args, archiverArgs...)
 
 	if artifact.Name != "" {
@@ -569,16 +595,20 @@ func (b *AbstractShell) writeUploadArtifact(w ShellWriter, info common.ShellScri
 		w.Notice("Uploading artifacts...")
 		w.Command(info.RunnerCommand, args...)
 	})
+
+	return true
 }
 
-func (b *AbstractShell) writeUploadArtifacts(w ShellWriter, info common.ShellScriptInfo, onSuccess bool) {
+func (b *AbstractShell) writeUploadArtifacts(w ShellWriter, info common.ShellScriptInfo, onSuccess bool) error {
 	if info.Build.Runner.URL == "" {
-		return
+		return common.ErrSkipBuildStage
 	}
 
 	b.writeExports(w, info)
 	b.writeCdBuildDir(w, info)
 
+	skipUploadArtifacts := true
+
 	for _, artifact := range info.Build.Artifacts {
 		if onSuccess {
 			if !artifact.When.OnSuccess() {
@@ -590,8 +620,16 @@ func (b *AbstractShell) writeUploadArtifacts(w ShellWriter, info common.ShellScr
 			}
 		}
 
-		b.writeUploadArtifact(w, info, artifact)
+		if b.writeUploadArtifact(w, info, artifact) {
+			skipUploadArtifacts = false
+		}
 	}
+
+	if skipUploadArtifacts {
+		return common.ErrSkipBuildStage
+	}
+
+	return nil
 }
 
 func (b *AbstractShell) writeAfterScript(w ShellWriter, info common.ShellScriptInfo) error {
@@ -603,12 +641,8 @@ func (b *AbstractShell) writeAfterScript(w ShellWriter, info common.ShellScriptI
 		}
 	}
 
-	if afterScriptStep == nil {
-		return nil
-	}
-
-	if len(afterScriptStep.Script) == 0 {
-		return nil
+	if afterScriptStep == nil || len(afterScriptStep.Script) == 0 {
+		return common.ErrSkipBuildStage
 	}
 
 	b.writeExports(w, info)
@@ -619,7 +653,7 @@ func (b *AbstractShell) writeAfterScript(w ShellWriter, info common.ShellScriptI
 	return nil
 }
 
-func (b *AbstractShell) writeArchiveCacheScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
+func (b *AbstractShell) writeArchiveCacheScript(w ShellWriter, info common.ShellScriptInfo) error {
 	b.writeExports(w, info)
 	b.writeCdBuildDir(w, info)
 
@@ -627,14 +661,12 @@ func (b *AbstractShell) writeArchiveCacheScript(w ShellWriter, info common.Shell
 	return b.cacheArchiver(w, info)
 }
 
-func (b *AbstractShell) writeUploadArtifactsOnSuccessScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
-	b.writeUploadArtifacts(w, info, true)
-	return
+func (b *AbstractShell) writeUploadArtifactsOnSuccessScript(w ShellWriter, info common.ShellScriptInfo) error {
+	return b.writeUploadArtifacts(w, info, true)
 }
 
-func (b *AbstractShell) writeUploadArtifactsOnFailureScript(w ShellWriter, info common.ShellScriptInfo) (err error) {
-	b.writeUploadArtifacts(w, info, false)
-	return
+func (b *AbstractShell) writeUploadArtifactsOnFailureScript(w ShellWriter, info common.ShellScriptInfo) error {
+	return b.writeUploadArtifacts(w, info, false)
 }
 
 func (b *AbstractShell) writeScript(w ShellWriter, buildStage common.BuildStage, info common.ShellScriptInfo) error {
@@ -643,17 +675,15 @@ func (b *AbstractShell) writeScript(w ShellWriter, buildStage common.BuildStage,
 		common.BuildStageGetSources:               b.writeGetSourcesScript,
 		common.BuildStageRestoreCache:             b.writeRestoreCacheScript,
 		common.BuildStageDownloadArtifacts:        b.writeDownloadArtifactsScript,
-		common.BuildStageUserScript:               b.writeUserScript,
 		common.BuildStageAfterScript:              b.writeAfterScript,
 		common.BuildStageArchiveCache:             b.writeArchiveCacheScript,
 		common.BuildStageUploadOnSuccessArtifacts: b.writeUploadArtifactsOnSuccessScript,
 		common.BuildStageUploadOnFailureArtifacts: b.writeUploadArtifactsOnFailureScript,
 	}
 
-	fn := methods[buildStage]
-	if fn == nil {
-		return errors.New("Not supported script type: " + string(buildStage))
+	fn, ok := methods[buildStage]
+	if !ok {
+		return b.writeUserScript(w, info, buildStage)
 	}
-
 	return fn(w, info)
 }
